@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2007 Sun Microsystems Inc.
+# Copyright 2004-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,15 +14,15 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__  = "$Id: focus_tracking_presenter.py 3782 2008-03-28 20:41:31Z eitani $"
-__version__   = "$Revision: 3782 $"
-__date__      = "$Date: 2008-03-28 16:41:31 -0400 (Fri, 28 Mar 2008) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__  = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import gobject
@@ -35,6 +35,7 @@ import braille
 import default
 import debug
 import input_event
+import orca
 import orca_state
 import presentation_manager
 import settings
@@ -73,7 +74,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._gidleLock      = threading.Lock()
         self.noFocusTimestamp = 0.0
 
-        self.setActiveScript(None)
+        self.setActiveScript(None, "__init__")
 
         # Initialize variable to make pylint happy.
         #
@@ -102,7 +103,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         - eventType: the event type.
         """
 
-        if self._listenerCounts.has_key(eventType):
+        if eventType in self._listenerCounts:
             self._listenerCounts[eventType] += 1
         else:
             self.registry.registerEventListener(self._enqueueEvent,
@@ -240,26 +241,31 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # package for now.
         #
         if (not script) and app and getattr(app, "toolkitName", None):
-            try:
-                debug.println(
-                    debug.LEVEL_FINE,
-                    "Looking for toolkit script %s.py..." % app.toolkitName)
-                module = __import__(app.toolkitName,
-                                    globals(),
-                                    locals(),
-                                    [''])
-                script = module.Script(app)
-                debug.println(debug.LEVEL_FINE,
-                              "...found %s.py" % name)
-            except ImportError:
-                debug.println(
-                    debug.LEVEL_FINE,
-                    "...could not find %s.py" % app.toolkitName)
-            except:
-                debug.printException(debug.LEVEL_SEVERE)
-                debug.println(
-                    debug.LEVEL_SEVERE,
-                    "While attempting to import %s" % app.toolkitName)
+            for package in scriptPackages:
+                if package:
+                    name = '.'.join((package, app.toolkitName))
+                else:
+                    name = app.toolkitName
+                try:
+                    debug.println(
+                        debug.LEVEL_FINE,
+                        "Looking for toolkit script %s.py..." % app.toolkitName)
+                    module = __import__(name,
+                                        globals(),
+                                        locals(),
+                                        [''])
+                    script = module.Script(app)
+                    debug.println(debug.LEVEL_FINE,
+                                  "...found %s.py" % name)
+                except ImportError:
+                    debug.println(
+                        debug.LEVEL_FINE,
+                        "...could not find %s.py" % app.toolkitName)
+                except:
+                    debug.printException(debug.LEVEL_SEVERE)
+                    debug.println(
+                        debug.LEVEL_SEVERE,
+                        "While attempting to import %s" % app.toolkitName)
 
         if not script:
             script = default.Script(app)
@@ -284,7 +290,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 self._defaultScript = default.Script(None)
                 self._registerEventListeners(self._defaultScript)
             script = self._defaultScript
-        elif self._knownScripts.has_key(app):
+        elif app in self._knownScripts:
             script = self._knownScripts[app]
         else:
             script = self._createScript(app)
@@ -293,7 +299,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         return script
 
-    def setActiveScript(self, newScript):
+    def setActiveScript(self, newScript, reason=None):
         """Set the new active script.
 
         Arguments:
@@ -322,6 +328,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         except:
             pass
 
+        if orca_state.activeScript:
+            debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: %s (reason=%s)" \
+                          % (orca_state.activeScript.name, reason))
+
     def _cleanupCache(self):
         """Looks for defunct accessible objects in the cache and removes them.
         """
@@ -399,7 +409,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 try:
                     debug.println(debug.LEVEL_FINEST,
                                   "Looking for settings at %s.py..." % name)
-                    if not self._knownAppSettings.has_key(name):
+                    if name not in self._knownAppSettings:
                         self._knownAppSettings[name] = \
                             __import__(name, globals(), locals(), [''])
                     reload(self._knownAppSettings[name])
@@ -534,6 +544,13 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 if state.contains(pyatspi.STATE_DEFUNCT):
                     debug.println(debug.LEVEL_FINEST,
                                   "IGNORING DEFUNCT OBJECT")
+                    if event.type.startswith("window:deactivate"):
+                        if orca_state.activeScript \
+                           and orca_state.activeScript.flatReviewContext:
+                            orca_state.activeScript.drawOutline(-1, 0, 0, 0)
+                            orca_state.activeScript.flatReviewContext = None
+                        orca.setLocusOfFocus(event, None)
+                        orca_state.activeWindow = None
                     return
 
             if (not debug.eventDebugFilter) \
@@ -591,6 +608,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             try:
                 state = event.source.getState()
                 if not state.contains(pyatspi.STATE_ICONIFIED):
+                    eType = event.type
+                    setNewActiveScript = eType == "window:activate"
+
+                    reason = None
+                    if not reason and setNewActiveScript:
+                        reason = "window:activate event"
 
                     # [[[TODO: WDW - HACK we look for frame that get
                     # focus: as a means to detect active scripts
@@ -600,18 +623,65 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # in use, and then issues a focus: event on the
                     # main window, which is a frame.]]]
                     #
-                    if (event.type == "window:activate") \
-                       or ((event.type.startswith("focus")) 
-                           and (event.source.getRole() == pyatspi.ROLE_FRAME)):
+                    setNewActiveScript = setNewActiveScript \
+                        or (eType.startswith("focus") \
+                            and (event.source.getRole() == pyatspi.ROLE_FRAME))
+
+                    if not reason and setNewActiveScript:
+                        reason = "frame received focus"
+
+                    # Added in a further check. We look for modal panels
+                    # that are now showing (such as gnome-screensaver-dialog).
+                    # See bug #530368 for more details.
+                    #
+                    setNewActiveScript = setNewActiveScript \
+                        or (eType.startswith("object:state-changed:showing")
+                            and (event.source.getRole() == pyatspi.ROLE_PANEL)
+                            and state.contains(pyatspi.STATE_MODAL))
+
+                    if not reason and setNewActiveScript:
+                        reason = "modal panel is showing"
+
+                    # Also, we might be running into a gnome-panel
+                    # applet, which is indicated by a host application
+                    # with no children.  See bug #536985.
+                    #
+                    #setNewActiveScript = setNewActiveScript \
+                    #    or (event.host_application \
+                    #        and len(event.host_application) == 0 \
+                    #        and orca_state.activeScript \
+                    #        and (orca_state.activeScript.app \
+                    #             != event.host_application))
+                    #
+                    #if not reason and setNewActiveScript:
+                    #    reason = "bizarre applet behavior"
+
+                    # Or, we might just be getting a focus event.  In this
+                    # case, assume the window has focus and we missed an
+                    # event for it somehow.
+                    #
+                    if not setNewActiveScript:
+                        if eType.startswith("focus") \
+                           or (eType.startswith("object:state-changed:focused")\
+                               and event.detail1):
+                            setNewActiveScript = \
+                                orca_state.activeScript \
+                                and event.host_application \
+                                and (orca_state.activeScript.app \
+                                     != event.host_application)
+
+                    if not reason and setNewActiveScript:
+                        reason = "object received focus"
+
+                    if setNewActiveScript:
                         # We'll let someone else decide if it's important
                         # to stop speech or not.
                         #speech.stop()
-                        debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
-                                      + orca_state.activeScript.name)
 
                         self.setActiveScript(
                             self.getScript(event.host_application or \
-                                              event.source.getApplication()))
+                                              event.source.getApplication()),
+                            reason)
 
                         # Load in the application specific settings for the
                         # app for this event (if found).
@@ -658,6 +728,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         - e: an at-spi event.
         """
 
+        asyncMode = settings.asyncMode
+
         if settings.debugEventQueue:
             if self._enqueueEventCount:
                 debug.println(debug.LEVEL_ALL,
@@ -679,7 +751,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             event = e
         elif isinstance(e, input_event.BrailleEvent):
             debug.println(debug.LEVEL_ALL,
-                          "----------> QUEUEING BRAILLE COMMAND %d" % e.event)
+                          "----------> QUEUEING BRAILLE COMMAND %s" \
+                          % repr(e.event))
             event = e
         else:
             if e.type in settings.ignoredEventsList:
@@ -730,6 +803,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_ALL,
                               "---------> QUEUEING EVENT %s" % e.type)
 
+            # Some toolkits (e.g., Java - see bug #531869) need to
+            # have their events processed immediately.
+            #
+            try:
+                if event.host_application.toolkitName \
+                    in settings.synchronousToolkits:
+                    asyncMode = False
+            except:
+                pass
+
         if event:
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
@@ -746,7 +829,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           ...put complete")
-            if settings.asyncMode and (not self._gidleId):
+            if asyncMode and (not self._gidleId):
                 if settings.gilSleepTime:
                     time.sleep(settings.gilSleepTime)
                 self._gidleId = gobject.idle_add(self._dequeueEvent)
@@ -759,7 +842,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_ALL,
                               "           ...released")
 
-            if not settings.asyncMode:
+            if not asyncMode:
                 self._dequeueEvent()
 
         if settings.debugEventQueue:
@@ -812,15 +895,15 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                               % (pressRelease, event.event_string))
             elif isinstance(event, input_event.BrailleEvent):
                 debug.println(debug.LEVEL_ALL,
-                              "DEQUEUED BRAILLE COMMAND %d <----------" \
-                              % event.event)
+                              "DEQUEUED BRAILLE COMMAND %s <----------" \
+                              % repr(event.event))
                 debug.println(debug.eventDebugLevel,
-                              "\nvvvvv PROCESS BRAILLE EVENT %d vvvvv"\
-                              % event.event)
+                              "\nvvvvv PROCESS BRAILLE EVENT %s vvvvv"\
+                              % repr(event.event))
                 self._processBrailleEvent(event)
                 debug.println(debug.eventDebugLevel,
-                              "\n^^^^^ PROCESS BRAILLE EVENT %d ^^^^^"\
-                              % event.event)
+                              "\n^^^^^ PROCESS BRAILLE EVENT %s ^^^^^"\
+                              % repr(event.event))
             else:
                 if (not debug.eventDebugFilter) \
                     or (debug.eventDebugFilter \
@@ -897,12 +980,14 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         Returns True if the event should be consumed.
         """
 
+        consume = False
         if orca_state.activeScript \
            and orca_state.activeScript.consumesKeyboardEvent(keyboardEvent):
-            self._enqueueEvent(keyboardEvent)
-            return True
-        else:
-            return False
+            consume = not orca_state.bypassNextCommand
+            if consume:
+                self._enqueueEvent(keyboardEvent)
+
+        return consume
 
     def processBrailleEvent(self, brailleEvent):
         """Called whenever a cursor key is pressed on the Braille display.
@@ -979,7 +1064,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         self._restoreAppStates()
 
-        self.setActiveScript(self.getScript(None))
+        self.setActiveScript(self.getScript(None), "activate")
 
         # Tell BrlTTY which commands we care about.
         #
@@ -1027,4 +1112,4 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._oldAppSettings = None
         self._defaultScript  = None
 
-        self.setActiveScript(None)
+        self.setActiveScript(None, "deactivate")
