@@ -33,9 +33,9 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py 4103 2008-08-15 11:07:31Z wwalker $"
-__version__   = "$Revision: 4103 $"
-__date__      = "$Date: 2008-08-15 07:07:31 -0400 (Fri, 15 Aug 2008) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -442,7 +442,7 @@ class Script:
         elif "default" in user_bindings_map:
             user_bindings = user_bindings_map["default"]
 
-        command = brailleEvent.event
+        command = brailleEvent.event["command"]
         consumes = False
         if user_bindings:
             consumes = command in user_bindings
@@ -481,7 +481,7 @@ class Script:
         #
         consumed = False
         user_bindings = None
-        command = brailleEvent.event
+        command = brailleEvent.event["command"]
 
         user_bindings_map = settings.brailleBindingsMap
         if self.name in user_bindings_map:
