@@ -19,9 +19,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: braille_generator.py 4221 2008-09-15 08:11:23Z wwalker $"
-__version__   = "$Revision: 4221 $"
-__date__      = "$Date: 2008-09-15 04:11:23 -0400 (Mon, 15 Sep 2008) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -51,6 +51,39 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         return braillegenerator.BrailleGenerator.\
             _getTextForRole(self, obj, role)
 
+    def _getBrailleRegionsForList(self, obj):
+        """Get the braille for a focused list.
+
+        Arguments:
+        - obj: the list
+
+        Returns a list where the first element is a list of Regions to
+        display and the second element is the Region which should get
+        focus.
+        """
+
+        self._debugGenerator("soffice: _getBrailleRegionsForList", obj)
+
+        if not obj.getState().contains(pyatspi.STATE_FOCUSABLE):
+            return braillegenerator.BrailleGenerator.\
+                       _getBrailleRegionsForList(self, obj)
+
+        text = ""
+        label = self._script.getDisplayedLabel(obj)
+        if not label:
+            label = obj.name
+        if label and len(label):
+            text = self._script.appendString(text, label)
+
+        text = self._script.appendString(text, self._getTextForRole(obj))
+        text = self._script.appendString(text,
+                                         self._getTextForRequiredObject(obj))
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
+
     def _getBrailleRegionsForTableCellRow(self, obj):
         """Get the braille for a table cell row or a single table cell
         if settings.readTableCellRow is False.
