@@ -20,15 +20,16 @@
 """Manages the default speech server for orca.  A script can use this
 as its speech server, or it can feel free to create one of its own."""
 
-__id__        = "$Id: speech.py 3904 2008-05-19 18:43:44Z wwalker $"
-__version__   = "$Revision: 3904 $"
-__date__      = "$Date: 2008-05-19 14:43:44 -0400 (Mon, 19 May 2008) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
 log = logging.getLogger("speech")
 
+import re
 import time
 
 import chnames
@@ -45,6 +46,12 @@ from orca_i18n import _           # for gettext support
 #
 _speechserver = None
 
+# regular expressions for multiCaseStrings
+#
+multiCaseReg1 = re.compile("([a-z]+)([A-Z])")
+multiCaseReg2 = re.compile("([A-Z][A-Z]+)([A-Z][a-z]+)")
+multiCaseReg3 = re.compile("([A-Z])([A-Z][a-z]+)")
+
 def getSpeechServerFactories():
     """Imports all known SpeechServer factory modules.  Returns a list
     of modules that implement the getSpeechServers method, which
@@ -154,6 +161,13 @@ def speak(text, acss=None, interrupt=True):
     if settings.silenceSpeech:
         return
 
+    if settings.speakMultiCaseStringsAsWords:
+        text = _processMultiCaseString(text)
+    if orca_state.activeScript and orca_state.usePronunciationDictionary:
+        text = orca_state.activeScript.adjustForPronunciation(text)
+    if settings.speakMultiCaseStringsAsWords:
+        text = _processMultiCaseString(text)
+
     logLine = "SPEECH OUTPUT: '" + text + "'"
     debug.println(debug.LEVEL_INFO, logLine)
     log.info(logLine)
@@ -245,11 +259,20 @@ def speakUtterances(utterances, acss=None, interrupt=True):
 
     if settings.silenceSpeech:
         return
-
-    for utterance in utterances:
-        logLine = "SPEECH OUTPUT: '" + utterance + "'"
+    i = 0
+    length = len(utterances)
+    while ( i < length ):
+        if settings.speakMultiCaseStringsAsWords:
+            utterances[i] = _processMultiCaseString(utterances[i])
+        if orca_state.activeScript and orca_state.usePronunciationDictionary:
+            utterances[i] = orca_state.activeScript.adjustForPronunciation(\
+                            utterances[i])
+        if settings.speakMultiCaseStringsAsWords:
+            utterances[i] = _processMultiCaseString(utterances[i])
+        logLine = "SPEECH OUTPUT: '" + utterances[i] + "'"
         debug.println(debug.LEVEL_INFO, logLine)
         log.info(logLine)
+        i = i + 1
 
     if _speechserver:
         _speechserver.speakUtterances(utterances,
@@ -345,3 +368,13 @@ def test():
                 server.shutdown()
             except:
                 debug.printException(debug.LEVEL_OFF)
+
+def _processMultiCaseString(string):
+    """Helper function, applies the regexes to split multiCaseStrings
+    to multiple words.
+    """
+
+    string = multiCaseReg1.sub('\\1 \\2', string)
+    string = multiCaseReg2.sub('\\1 \\2', string)
+    string = multiCaseReg3.sub('\\1 \\2', string)    
+    return string
