@@ -19,9 +19,9 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 4611 2009-02-27 16:01:04Z wwalker $"
-__version__   = "$Revision: 4611 $"
-__date__      = "$Date: 2009-02-27 11:01:04 -0500 (Fri, 27 Feb 2009) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -158,6 +158,8 @@ class WhereAmI:
         if basicOnly:
             self._speakObjDescription(obj)
 
+        self._lastAttributeString = ""
+
         return True
 
     def _speakCheckBox(self, obj, basicOnly):
@@ -510,10 +512,11 @@ class WhereAmI:
         Gaim, gedit, OpenOffice Writer and Terminal
         """
 
-        cell = self._script.getAncestor(obj,
-                                        [pyatspi.ROLE_TABLE_CELL],
-                                        [pyatspi.ROLE_FRAME])
-        if cell and not self._script.isLayoutOnly(cell.parent):
+        ancestor = self._script.getAncestor(obj,
+                                            [pyatspi.ROLE_TABLE_CELL,
+                                             pyatspi.ROLE_LIST_ITEM],
+                                            [pyatspi.ROLE_FRAME])
+        if ancestor and not self._script.isLayoutOnly(ancestor.parent):
             # [[[TODO: WDW - we handle ROLE_ENTRY specially here because
             # there is a bug in getRealActiveDescendant: it doesn't dive
             # deep enough into the hierarchy (see comment #12 of bug
@@ -521,8 +524,11 @@ class WhereAmI:
             # more comfortable with mucking around with
             # getRealActiveDescendant.]]]
             #
-            if obj.getRole() != pyatspi.ROLE_ENTRY:
-                return self._speakTableCell(cell, basicOnly)
+            if ancestor.getRole() == pyatspi.ROLE_TABLE_CELL:
+                if obj.getRole() != pyatspi.ROLE_ENTRY:
+                    return self._speakTableCell(ancestor, basicOnly)
+            else:
+                return self._speakListItem(ancestor, basicOnly)
 
         utterances = []
         text = self._getObjLabel(obj)
@@ -745,6 +751,19 @@ class WhereAmI:
             # view (i.e., how many ancestors a node has).
             #
             utterances.append(_("tree level %d") % (level + 1))
+        else:
+            nestingLevel = 0
+            parent = obj.parent
+            while parent.parent.getRole() == pyatspi.ROLE_LIST:
+                nestingLevel += 1
+                parent = parent.parent
+            if nestingLevel:
+                # Translators: this represents a list item in a document.
+                # The nesting level is how 'deep' the item is (e.g., a
+                # level of 2 represents a list item inside a list that's
+                # inside another list).
+                #
+                utterances.append(_("Nesting level %d") % nestingLevel)
 
         getTutorial = self._script.tutorialGenerator.getTutorial
         utterances.extend(getTutorial(obj, False, forceMessage=True))
@@ -1652,49 +1671,69 @@ class WhereAmI:
             debug.println(self._debugLevel,
                           "line attribs <%s>" % (attribs))
             if attribs:
-                newLine += " ; "
+                if newLine:
+                    newLine += " ; "
                 newLine += attribs
                 newLine += " "
 
             newLine += line[i]
             textOffset += 1
 
+        attribs = self._getAttributesForChar(obj,
+                                             text,
+                                             startOffset,
+                                             line,
+                                             0,
+                                             ["paragraph-style"])
+        if attribs:
+            if newLine:
+                newLine += " ; "
+            newLine += attribs
+
         debug.println(self._debugLevel, "newLine: <%s>" % (newLine))
 
         return newLine
 
-    def _getAttributesForChar(self, obj, text, textOffset, line, lineIndex):
+    def _getAttributesForChar(self,
+                              obj,
+                              text,
+                              textOffset,
+                              line,
+                              lineIndex,
+                              keys=["style", "weight", "underline"]):
 
-        keys = [ "style", "weight", "underline" ]
         attribStr = ""
-        charAttributes = text.getAttributes(textOffset)
 
+        defaultAttributes = text.getDefaultAttributes()
+        attributesDictionary = self._stringToDictionary(defaultAttributes)
+
+        charAttributes = text.getAttributes(textOffset)
         if charAttributes[0]:
             charDict = self._stringToDictionary(charAttributes[0])
-            debug.println(self._debugLevel,
-                          "charDict: %s" % (charDict))
+            for key in charDict.keys():
+                attributesDictionary[key] = charDict[key]
 
+        if attributesDictionary:
             for key in keys:
                 localizedKey = text_attribute_names.getTextAttributeName(key)
-                if key in charDict:
-                    attribute = charDict[key]
+                if key in attributesDictionary:
+                    attribute = attributesDictionary[key]
                     localizedValue = \
                         text_attribute_names.getTextAttributeName(attribute)
                     if attribute:
                         # If it's the 'weight' attribute and greater than 400,
                         # just speak it as bold, otherwise speak the weight.
                         #
-                        if key == "weight" and int(attribute) > 400:
-                            attribStr += " "
-                            # Translators: bold as in the font sense.
-                            #
-                            attribStr += _("bold")
-
+                        if key == "weight":
+                            if int(attribute) > 400:
+                                attribStr += " "
+                                # Translators: bold as in the font sense.
+                                #
+                                attribStr += _("bold")
                         elif key == "underline":
                             if attribute != "none":
                                 attribStr += " "
                                 attribStr += localizedKey
-
                         elif key == "style":
                             if attribute != "normal":
                                 attribStr += " "
@@ -1712,8 +1751,9 @@ class WhereAmI:
                 #
                 attribStr += _("link")
 
-            debug.println(self._debugLevel,
-                          "char <%s>: %s" % (line[lineIndex], attribStr))
+            if line:
+                debug.println(self._debugLevel,
+                              "char <%s>: %s" % (line[lineIndex], attribStr))
 
         # Only return attributes for the beginning of an attribute run.
         if attribStr != self._lastAttributeString:
